require "spec_helper"

class Book
  include ActiveAttr::Model

  attr_accessor :state, :description
end

class Shelf
  include ActiveAttr::Model
end

class Ability
  include CanCan::Ability

  def initialize
    canf :update, Book, :title

    canf :update, Book, :author, state: 'unpublished'

    canf :update, Book, :description do |book|
      book.state == 'unpublished'
    end

    canf [:create, :update], Shelf, [:name, :position]
  end
end

describe "CanField::CanCan" do
  describe "::AbilityAdditions" do
    subject{ ability }
    let(:ability) {Ability.new()}

    describe "#canf" do
      it { should     be_able_to(:_cf_update_fl_title,  Book.new) }
      it { should_not be_able_to(:_cf_update_fl_author, Book.new) }

      it { should     be_able_to(:update, Book.new) }
      it { should_not be_able_to(:create, Book.new) }

      it { should be_able_to(:_cf_update_fl_name,     Shelf.new) }
      it { should be_able_to(:_cf_update_fl_position, Shelf.new) }
      it { should be_able_to(:_cf_create_fl_name,     Shelf.new) }
      it { should be_able_to(:_cf_create_fl_position, Shelf.new) }

      it { should     be_able_to(:update, Shelf.new)}
      it { should     be_able_to(:create, Shelf.new)}
      it { should_not be_able_to(:read,   Shelf.new)}

      it { should     be_able_to(:_cf_update_fl_author, Book.new(state: 'unpublished')) }
      it { should_not be_able_to(:_cf_update_fl_author, Book.new) }

      it { should     be_able_to(:_cf_update_fl_description, Book.new(state: 'unpublished')) }
      it { should_not be_able_to(:_cf_update_fl_description, Book.new) }
    end
  end
end